package de.opalpiraten.scenepainter.gui;
import java.awt.*;
import java.io.*;    
import java.util.*;
import javax.swing.*;
import java.awt.event.*;  
import javax.swing.event.*;
import java.beans.*;
import java.net.*;
import javax.swing.tree.*;
import com.threerings.media.image.*;
import com.threerings.media.tile.*;


import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * This Editor is the GUI frontend for the TileSceneElement class
 */
public class TileSceneElementEditor extends JPanel implements 
ListSelectionListener, TreeSelectionListener, ChangeListener, SceneListener {

  /**
   * The name of the root node for the tile tree
   */
  private static final String TILEROOT ="Tiles";
  
  /**
   * The name of the root node for the color tree
   */
  private static final String COLORROOT = "Colors";

  /**
   * List the resourcenames in a more userfriendly way
   */
  private JTree rsrctree;
  
  /**
   * Lists the colors available for a Tile
   */
  private JTree coltree;
  
  /**
   * The Index number of the Tile in the tileset.
   */
  private JSlider index = new JSlider(0,0);
  
  /**
   * Reference to the scene showing
   */
  private Scene scene;
  
  /**
   * Reference to the current project
   */
  private Project project;
  
  /**
   * Reference to the container, this editor resides in
   */
  private SceneElementEditor parent;
  
  public TileSceneElementEditor(SceneElementEditor see) {
    parent = see;
    DefaultMutableTreeNode top_rsrc = new DefaultMutableTreeNode(TILEROOT);
     
    rsrctree = new JTree(top_rsrc);
    coltree = new JTree(new DefaultMutableTreeNode("Colors"));
    coltree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
    
    String[] names = Core.nenyabrowser.getTileNames();
    for (int i=0;i<names.length;i++) Util.sortIntoTree(top_rsrc,names[i].split("/"));
    if (names.length>0) rsrctree.addTreeSelectionListener(this);
    
    rsrctree.expandPath(new TreePath(top_rsrc));
    
    JScrollPane jsp0 = new JScrollPane(rsrctree);
    JScrollPane jsp1 = new JScrollPane(coltree);
    JTabbedPane split = new JTabbedPane(JTabbedPane.BOTTOM);
    split.add(Dict.get("gui.tilesceneelementeditor.split.tiles"),jsp0);
    split.add(Dict.get("gui.tilesceneelementeditor.split.colors"),jsp1);
    index.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.tilesceneelementeditor.index")));
    
    split.setPreferredSize(split.getPreferredSize());
    index.addChangeListener(this);
    index.setPaintTicks(true);
    index.setSnapToTicks(true);
    index.setPaintLabels(true);
    index.setMajorTickSpacing(1);
    
    setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
    add(split);
    add(index);
  }
  
  public void stateChanged(ChangeEvent e) {
    Object src = e.getSource();
    if (src==index) {
      try {
        SceneElement se = scene.getSelected();
        ((TileSceneElement)se).setIndex(index.getValue());
        scene.replace(scene.getSelectedIndex(),se);
      }
      catch (Exception exp) {
        // lazy way to skip sanity checks
      }
    }
  }
  
  public void replaceScene(Scene s) {
    if (scene!=null) scene.removeSceneListener(this);
    scene=s;
    scene.addSceneListener(this);
    sceneChanged(s);
  }
  
  public void sceneChanged (Scene s) {
    // Important: Take the Listeners offline while editing, this
    // method becomes reentrant otherwise
    index.removeChangeListener(this);
    coltree.removeTreeSelectionListener(this);
    
    try {
      // Order is very important here!
      TileSceneElement tse = (TileSceneElement)s.getSelected();
      index.setValue(tse.getIndex());
      TileManager tm = Core.nenyabrowser.getTileManager();
      TileSet ts = tm.getTileSet(tse.getResource());
      index.setMaximum(ts.getTileCount()-1);
      
      coltree.setModel(new DefaultTreeModel(null)); //new DefaultMutableTreeNode(COLORROOT)));
      RecolorableTileSet rts = (RecolorableTileSet)ts;
      String tmp[] = rts.getColorizations();
      ColorPository pos = project.getColorPository();
      Vector paths = new Vector();
      for (int x=0;x<tmp.length;x++) {
        // Hopefully nobody ever gets the bright idea of using a "/" in a name
        ColorPository.ColorRecord[] crec = pos.enumerateColors(tmp[x]);
        for (int y=0;y<crec.length;y++) {
          paths.add(tmp[x]+"/"+crec[y].name);
        }
      }
      tmp =new String[paths.size()];
      paths.copyInto(tmp);
      Arrays.sort(tmp);
      DefaultMutableTreeNode top = new DefaultMutableTreeNode(COLORROOT);
      for (int i=0;i<tmp.length;i++) Util.sortIntoTree(top,tmp[i].split("/"));
      DefaultTreeModel dtm = new DefaultTreeModel(top);
      coltree.setModel(dtm);
      
      Hashtable hash = tse.getZations();
      if (hash==null) hash=new Hashtable();
      Enumeration e = hash.keys();
      while (e.hasMoreElements()) {
        String k= (String)e.nextElement();
        String v= (String)hash.get(k);
        // Oh the agony ... Finding the nodes to select
        for (int x=0;x<top.getChildCount();x++) {
          TreeNode tn0 = top.getChildAt(x);
          for(int y=0;y<tn0.getChildCount();y++) {
            TreeNode tn1 = tn0.getChildAt(y);
            if (k.equals(tn0.toString()) && v.equals(tn1.toString())) {
              // Remember: This is unsafe, it relies on the toString()
              // operation, which is not guarantied to deliever the original
              // name
              // System.err.println(top+"/"+tn0+"/"+tn1);
              Object o[] = {top,tn0,tn1};
              TreePath tp = new TreePath(o);
              coltree.addSelectionPath(tp);
              coltree.expandPath(tp);
            }
          }
        }
      }
    }
    catch (Exception e) {
      //e.printStackTrace();
      // Lazy way to skip all the sanity checks
    }
    coltree.addTreeSelectionListener(this);
    index.addChangeListener(this);
  }
  
  public void replaceProject(Project p) {
    project =p;
  }
  
  public void valueChanged(ListSelectionEvent e) {
  }
  
  public void valueChanged(TreeSelectionEvent e) {
    Object src = e.getSource();
    
    if (src == rsrctree) {
      Object path[] = rsrctree.getSelectionPath().getPath();
      if ((path==null || !((TreeNode)path[path.length-1]).isLeaf())) return;
    
      StringBuffer sb = new StringBuffer();
      for(int i=1;i<path.length;i++) {
        sb.append(path[i]);
        if (i<path.length-1) sb.append("/");
      }
    
      TileSceneElement tse = new TileSceneElement();
      tse.setResource(sb.toString());
      tse.setProject(project);
      
      
      if (parent.isCreating() || scene.getSelectedIndex()<0) {
        scene.add(tse);
        scene.setSelectedIndex(0);
      }
      else {
        SceneElement se = scene.getSelected();
        if (se!=null) {
          tse.setPosition(se.getPosition());
          tse.setAlpha(se.getAlpha());
        }
        scene.replace(scene.getSelectedIndex(),tse);
      }
      rsrctree.removeTreeSelectionListener(this);
      rsrctree.clearSelection();
      rsrctree.addTreeSelectionListener(this);
    }
    
    if (src==coltree) {
      try {
        TileSceneElement tse = (TileSceneElement)scene.getSelected();
        Hashtable colz = tse.getZations();
        if (colz== null) colz=new Hashtable();
        TreePath tp[] = coltree.getSelectionPaths();
        if (tp!=null) {
          for (int i=0;i<tp.length;i++) {
            // OUCH!
            // Hope, that the definition of DefaultMutableTreeNode.toString()
            // never changes, because otherwise this will cause big trouble!
            // Note: If it does, implement a Treenode, that allows the retrieval
            // of the node object.
            if (tp[i].getPath().length==3)
            colz.put(tp[i].getPathComponent(1).toString(),tp[i].getPathComponent(2).toString());
            else colz = new Hashtable();
          }
        }
        tse.setZations(colz);
        scene.replace(scene.getSelectedIndex(),tse);
      }
      catch (Exception exp) {
        // Again: be lazy about sanity checks
      }
    }
  }
}
